/*
 * SegmentedControl.cpp
 *
 *  Created on: 2010.09.09.
 *      Author: Karma
 */

#include "SegmentedControl.h"
#include "Segment.h"
#include <FMedia.h>

using namespace Osp::Base;
using namespace Osp::Graphics;
using namespace Osp::Media;
using namespace Osp::Ui;

Bitmap *SegmentedControl::__bg(NULL);

void
SegmentedControl::PreloadBitmaps()
{
	Segment::PreloadBitmaps();
	GetBackgroundBitmap();
}

SegmentedControl::SegmentedControl() :
	__selectedSegmentId(-1), __selectedSegmentIndex(-1), __canvas(NULL)
{
}

SegmentedControl::~SegmentedControl() {
	__listeners.RemoveAll();
	__segmentsById.RemoveAll();
	__segmentsByIndex.RemoveAll(false);
}

result
SegmentedControl::Construct(const Rectangle &bounds)
{
	Panel::Construct(bounds);
	__listeners.Construct(10);
	__segmentsById.Construct(10, 0.75);
	__segmentsByIndex.Construct(10);
	return E_SUCCESS;
}

result
SegmentedControl::AddSegment(const String &caption, int actionId)
{
	Segment *segment = new Segment(*this);
	segment->Construct(caption, actionId);
	AddControl(*segment);
	__segmentsById.Add(actionId, segment);
	__segmentsByIndex.Add(*segment);

	if (__selectedSegmentId == -1)
	{
		__selectedSegmentId = actionId;
		__selectedSegmentIndex = 0;
		segment->SetSelectedState(true);
	}

	LayoutSegments();

	return E_SUCCESS;
}

result
SegmentedControl::SetSelectedSegmentById(int actionId)
{
	result r = E_SUCCESS;

	if (__selectedSegmentId != actionId)
	{
		if (__selectedSegmentId != -1)
		{
			Segment *segment = NULL;
			r = __segmentsById.GetValue(__selectedSegmentId, segment);
			if (r == E_SUCCESS)
			{
				segment->SetSelectedState(false);
			}
			else
			{
				AppLog("Consistency error in SegmentedControl, segment %d not found!", actionId);
			}
		}

		Segment *newSegment = NULL;
		r = __segmentsById.GetValue(actionId, newSegment);
		TryReturn(r == E_SUCCESS, r, "Segment %d not found! [%S]", actionId, GetErrorMessage(r));
		newSegment->SetSelectedState(true);
		__selectedSegmentId = actionId;
		r = __segmentsByIndex.IndexOf(*newSegment, __selectedSegmentIndex);
		TryReturn(r == E_SUCCESS, r, "Consistency error in SegmentedControl, segment %d not in ArrayList! [%S]", actionId, GetErrorMessage(r));
	}

	return r;
}

int
SegmentedControl::GetSelectedSegmentId()
{
	return __selectedSegmentId;
}

result
SegmentedControl::SetSelectedSegmentByIndex(int index)
{
	result r = E_SUCCESS;

	if (__selectedSegmentIndex != index)
	{
		if (__selectedSegmentIndex != -1)
		{
			Segment *segment = static_cast<Segment *>(__segmentsByIndex.GetAt(__selectedSegmentIndex));
			if (segment)
			{
				segment->SetSelectedState(false);
			}
			else
			{
				AppLog("Consistency error in SegmentedControl, segment no. %d not found!", index);
			}
		}

		Segment *newSegment = static_cast<Segment *>(__segmentsByIndex.GetAt(index));
		TryReturn(newSegment, GetLastResult(), "Segment no. %d not found! [%S]", index, GetErrorMessage(GetLastResult()));
		newSegment->SetSelectedState(true);
		__selectedSegmentIndex = index;
		__selectedSegmentId = newSegment->GetActionId();
	}

	return r;
}

int
SegmentedControl::GetSelectedSegmentIndex()
{
	return __selectedSegmentIndex;
}

void
SegmentedControl::AddActionEventListener(IActionEventListener &listener)
{
	__listeners.Add(&listener);
}

void
SegmentedControl::RemoveActionEventListener(IActionEventListener &listener)
{
	__listeners.Remove(&listener);
}

result
SegmentedControl::OnDraw()
{
	result r = E_SUCCESS;
	Canvas *pCanvas = GetCanvasN();
	if (pCanvas)
	{
		r = pCanvas->DrawNinePatchedBitmap(GetBounds(), *GetBackgroundBitmap());
		delete pCanvas;
	}
	else
	{
		r = GetLastResult();
	}

	return r;
}

Bitmap *
SegmentedControl::GetBackgroundBitmap()
{
	if (!__bg)
	{
		result r;
		Image image;
		r = image.Construct();
		TryReturn(r == E_SUCCESS, NULL, "Image.Construct failed [%S]", GetErrorMessage(r));

		Bitmap *tmp = image.DecodeN(L"/Res/Segm_Background_9P.png", BITMAP_PIXEL_FORMAT_ARGB8888);
		if (tmp)
		{
			__bg = tmp;
		}
		else
		{
			TryReturn(false, NULL, "Image.DecodeN failed [%S]", GetErrorMessage(GetLastResult()));
		}
	}

	return __bg;
}

void
SegmentedControl::OnActionPerformed(const Control &source, int actionId)
{
	result r = SetSelectedSegmentById(actionId);
	if (r == E_SUCCESS)
	{
		int count = __listeners.GetCount();
		for (int i = 0; i < count; i++)
		{
			IActionEventListener *listener = NULL;
			__listeners.GetAt(i, listener);
			listener->OnActionPerformed(*this, actionId);
		}
	}
}

void
SegmentedControl::LayoutSegments()
{
	Rectangle bounds = GetBounds();

	// add some padding
	bounds.x += 12;
	bounds.width -= 24;
	bounds.y += 8;
	bounds.height -= 16;

	int count = __segmentsByIndex.GetCount();
	bounds.width /= count;

	for (int i = 0; i < count; i++)
	{
		Segment *segment = static_cast<Segment *>(__segmentsByIndex.GetAt(i));
		segment->SetBounds(bounds);
		bounds.x += bounds.width;

		int first = (i == 0) ? 1 : 0;
		int last = (i == count - 1) ? 2 : 0;

		segment->SetPositionId(first + last);
	}

	RequestRedraw(true);
}
